
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "gp-enum-types.h"
#include "gp-applet.h"

GType
gp_applet_flags_get_type (void)
{
  static gsize g_enum_type_id = 0;
  static const GFlagsValue values[] =
    {
      { GP_APPLET_FLAGS_NONE, "GP_APPLET_FLAGS_NONE", "none" },
      { GP_APPLET_FLAGS_EXPAND_MAJOR, "GP_APPLET_FLAGS_EXPAND_MAJOR", "expand-major" },
      { GP_APPLET_FLAGS_EXPAND_MINOR, "GP_APPLET_FLAGS_EXPAND_MINOR", "expand-minor" },
      { GP_APPLET_FLAGS_HAS_HANDLE, "GP_APPLET_FLAGS_HAS_HANDLE", "has-handle" },
      { 0, NULL, NULL }
    };

  if (g_once_init_enter (&g_enum_type_id))
    {
      const char *string;
      GType id;

      string = g_intern_static_string ("GpAppletFlags");
      id = g_flags_register_static (string, values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
#include "gp-lockdown.h"

GType
gp_lockdown_flags_get_type (void)
{
  static gsize g_enum_type_id = 0;
  static const GFlagsValue values[] =
    {
      { GP_LOCKDOWN_FLAGS_NONE, "GP_LOCKDOWN_FLAGS_NONE", "none" },
      { GP_LOCKDOWN_FLAGS_APPLET, "GP_LOCKDOWN_FLAGS_APPLET", "applet" },
      { GP_LOCKDOWN_FLAGS_FORCE_QUIT, "GP_LOCKDOWN_FLAGS_FORCE_QUIT", "force-quit" },
      { GP_LOCKDOWN_FLAGS_LOCKED_DOWN, "GP_LOCKDOWN_FLAGS_LOCKED_DOWN", "locked-down" },
      { GP_LOCKDOWN_FLAGS_COMMAND_LINE, "GP_LOCKDOWN_FLAGS_COMMAND_LINE", "command-line" },
      { GP_LOCKDOWN_FLAGS_LOCK_SCREEN, "GP_LOCKDOWN_FLAGS_LOCK_SCREEN", "lock-screen" },
      { GP_LOCKDOWN_FLAGS_LOG_OUT, "GP_LOCKDOWN_FLAGS_LOG_OUT", "log-out" },
      { GP_LOCKDOWN_FLAGS_USER_SWITCHING, "GP_LOCKDOWN_FLAGS_USER_SWITCHING", "user-switching" },
      { 0, NULL, NULL }
    };

  if (g_once_init_enter (&g_enum_type_id))
    {
      const char *string;
      GType id;

      string = g_intern_static_string ("GpLockdownFlags");
      id = g_flags_register_static (string, values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* Generated data ends here */

